import React, { useEffect, useRef, useState } from "react";
import {
  StyleSheet,
  View,
  Text,
  TouchableOpacity,
  Image,
  Button,
  LogBox,
  StatusBar,
  Linking,
  BackHandler,
  useWindowDimensions,
  Dimensions,
  FlatList,
  Animated,
  Alert,
} from "react-native";
import Header from "react-native-digitcamera/src/Screens/Header";
// import FirstRoute from "./FirstRoute";
// import SecondRoute from "./SecondRoute";
import { TabView, SceneMap, TabBar } from "react-native-tab-view";
import {
  TextInput,
  PrimaryButton,
  NewRoundButton,
  Spinner,
  NewRadioButton,
} from "../../common/ThemedComponents";
import NetInfo from "@react-native-community/netinfo";
import { checkInternetSpeed } from "react-native-digitcamera/src/Screens/OfflineFlow/CommonFunc";
import { useIsFocused } from "@react-navigation/native";
import { ProgressBar } from "react-native-paper";
import MCIcon from "react-native-vector-icons/MaterialCommunityIcons";
import Database from "../../../../../Components/common/DAsyncStorage";
import style from "react-native-digitcamera/src/common/CommonStyle";
import Colors from "react-native-digitcamera/src/common/Colors";

//Constants
const SCREEN_WIDTH = Math.round(Dimensions.get("window").width);
const SCREEN_HEIGHT = Math.round(Dimensions.get("window").height);
var docProgress = 0;
var interval;
// const MediaStatusSchemaX = [
//   {
//     activityId: "string",
//     activityTypeId: "string",
//     policyNumber: "23876847",
//     registrationNumber: "string",
//     activityStatus: "PENDING", // PENDING, IN-PROGRESS, COMPLETED
//     mode: "ONLINE", // ONLINE, OFFLINE
//     mediaUpload: MediaUpload,
//   },
//   {
//     activityId: "string",
//     activityTypeId: "string",
//     policyNumber: "0932898",
//     registrationNumber: "string",
//     activityStatus: "IN-PROGRESS", // PENDING, IN-PROGRESS, COMPLETED
//     mode: "OFFLINE", // ONLINE, OFFLINE
//     mediaUpload: MediaUpload,
//   },
//   {
//     activityId: "string",
//     activityTypeId: "string",
//     policyNumber: "8927398",
//     registrationNumber: "string",
//     activityStatus: "IN-PROGRESS", // PENDING, IN-PROGRESS, COMPLETED
//     mode: "OFFLINE", // ONLINE, OFFLINE
//     mediaUpload: MediaUpload,
//   },
//   {
//     activityId: "string",
//     activityTypeId: "string",
//     policyNumber: "9823798",
//     registrationNumber: "string",
//     activityStatus: "COMPLETED", // PENDING, IN-PROGRESS, COMPLETED
//     mode: "ONLINE", // ONLINE, OFFLINE
//     mediaUpload: MediaUpload,
//   },
//   {
//     activityId: "string",
//     activityTypeId: "string",
//     policyNumber: "2398732",
//     registrationNumber: "string",
//     activityStatus: "COMPLETED", // PENDING, IN-PROGRESS, COMPLETED
//     mode: "ONLINE", // ONLINE, OFFLINE
//     mediaUpload: MediaUpload,
//   },
// ];
var MediaStatusSchema = [];
var uploadedArray = [];
var unsubscribeNetInfo;
const MediaUpload = {
  status: "string", //uploading,failed,uploaded
  uri: "string",
  imageName: "string",
};

const _emptyItem = () => {
  return (
    <View style={{ flex: 1, justifyContent: "center", alignItems: "center" }}>
      <Text style={[style.muliRegLG, { fontSize: 14 }]}>
        No in progress data available
      </Text>
    </View>
  );
};

const ActivityScreen = ({ navigation, route }) => {
  const layout = useWindowDimensions();
  const [isInternetConnected, SetIsInternetConnected] = useState(true);
  const [isStatus, setIsStatus] = useState(true);
  var focused = useIsFocused();
  console.log("focused ", focused);
  const [index, setIndex] = React.useState(0);
  const [uploadTimeAndSpeed, setUploadTimeAndSpeed] = useState({
    uploadSpeed: 0,
    secondsRemaining: 0,
  });
  // const car = require("../common/Image/new/four_wheeler.png");
  const car = require("../../common/Image/new/four_wheeler.png");

  const [routes] = React.useState([
    { key: "first", title: "In-Progress" },
    { key: "second", title: "Completed " },
  ]);

  const FirstRoute = () => (
    <View style={styles.main}>
      <FlatList
        data={
          MediaStatusSchema.filter(
            (item) => item.activityStatus === "IN-PROGRESS"
          ).length === 0
            ? []
            : MediaStatusSchema
        }
        keyExtractor={(item, index) => index.toString()}
        ListEmptyComponent={_emptyItem}
        renderItem={({ item, index }) => {
          if (item.activityStatus == "IN-PROGRESS") {
            //do something

            return (
              <View
                style={{
                  flexDirection: "row",
                  padding: 16,
                  borderBottomColor: "#e9ebed",
                  borderBottomWidth: 0.4,
                  justifyContent: "space-evenly",
                  alignItems: "center",
                }}
              >
                <View
                  style={{
                    flex: 0.3,
                    justifyContent: "center",
                    alignItems: "center",
                  }}
                >
                  <Image
                    source={car}
                    resizeMode={"contain"}
                    style={{
                      aspectRatio: 1,
                      alignSelf: "center",
                      borderRadius: 5,
                    }}
                  />
                </View>
                <View
                  style={{
                    flex: 0.6,
                    flexDirection: "column",
                    padding: 8,
                  }}
                >
                  <View
                    style={{
                      flexDirection: "row",
                      paddingVertical: 5,
                    }}
                  >
                    <Text style={[style.muliBoldDB, { fontSize: 14 }]}>
                      {item.regNumber}
                      {item.policyNumber}
                    </Text>
                  </View>

                  <ProgressBar
                    progress={
                      isNaN(item.imageProgress) ||
                      item.imageProgress == undefined ||
                      item.imageProgress == null
                        ? 0
                        : item.imageProgress
                    }
                    color={"#FFBB00"}
                    style={{
                      height: 4,
                      width: "100%",
                      borderRadius: 12,
                      backgroundColor: "#EAEAEA",
                    }}
                  />

                  {isInternetConnected ? (
                    <View
                      style={{
                        flexDirection: "row",
                        paddingVertical: 5,
                        justifyContent: "space-between",
                      }}
                    >
                      <Text
                        style={[
                          style.muliBoldDB,
                          {
                            flexDirection: "column",
                            color: "#10B363",
                            fontSize: 11,
                          },
                        ]}
                      >
                        Uploading
                      </Text>
                      <Text
                        style={{
                          flexDirection: "column",
                          color: "#000000",
                          fontSize: 10,
                        }}
                      >
                        <MCIcon
                          name="wifi"
                          size={14}
                          color="black"
                          style={{ marginRight: 4 }}
                        />
                        {getSpeedInMbps(uploadTimeAndSpeed?.uploadSpeed)}
                      </Text>
                      <Text
                        style={{
                          flexDirection: "column",
                          color: "#999999",
                          fontSize: 10,
                          fontStyle: "italic",
                        }}
                      >
                        {getTimeInMinAndSec(
                          uploadTimeAndSpeed?.secondsRemaining
                        )}{" "}
                        left
                      </Text>
                    </View>
                  ) : (
                    <View
                      style={{
                        flex: 0.33,
                        flexDirection: "row",
                        padding: 5,
                      }}
                    >
                      <Text
                        style={{
                          flex: 0.5,
                          flexDirection: "column",
                          color: "#999999",
                          fontSize: 12,
                        }}
                      >
                        Upload Paused
                      </Text>

                      <Text
                        style={{
                          flex: 0.5,
                          flexDirection: "column",
                          color: "#999999",
                          fontSize: 10,
                        }}
                      >
                        <MCIcon
                          name="wifi-off"
                          size={16}
                          color="red"
                          style={{ marginRight: 4 }}
                        />
                        Poor Network
                      </Text>
                    </View>
                  )}
                </View>
              </View>
            );
          }
        }}
        contentContainerStyle={{ flexGrow: 0.9 }}
      />
    </View>
  );

  const SecondRoute = () => (
    <View style={styles.main}>
      <FlatList
        data={
          MediaStatusSchema.filter(
            (item) => item.activityStatus === "COMPLETED"
          ).length === 0
            ? []
            : MediaStatusSchema
        }
        keyExtractor={(item, index) => index.toString()}
        ListEmptyComponent={_emptyItem}
        renderItem={({ item, index }) => {
          if (item.activityStatus == "COMPLETED") {
            //do something
            return (
              <View
                style={{
                  flexDirection: "row",
                  padding: 16,
                  borderBottomColor: "#e9ebed",
                  borderBottomWidth: 0.4,
                  justifyContent: "space-evenly",
                  alignItems: "center",
                }}
              >
                <View
                  style={{
                    flex: 0.3,
                    justifyContent: "center",
                    alignItems: "center",
                  }}
                >
                  <Image
                    source={car}
                    resizeMode={"contain"}
                    style={{
                      aspectRatio: 1,
                      alignSelf: "center",
                      borderRadius: 5,
                    }}
                  />
                </View>
                <View
                  style={{
                    flex: 0.6,
                    flexDirection: "column",
                    paddingVertical: 8,
                  }}
                >
                  <View
                    style={{
                      flexDirection: "row",
                      paddingVertical: 5,
                    }}
                  >
                    <Text style={[style.muliboldBB]}>
                      {item.regNumber + " "} {item.policyNumber}
                    </Text>
                  </View>
                  <ProgressBar
                    progress={
                      // isNaN(item.imageProgress) ||
                      // item.imageProgress == undefined ||
                      // item.imageProgress == null
                      //   ? 0
                      //   : item.imageProgress
                      1
                    }
                    color={"#FFBB00"}
                    style={{
                      height: 4,
                      width: "100%",
                      borderRadius: 12,
                      backgroundColor: "#EAEAEA",
                    }}
                  />

                  <View
                    style={{
                      // flex: 0.33,
                      flexDirection: "row",
                      justifyContent: "space-between",
                      alignItems: "center",
                      paddingVertical: 5,
                    }}
                  >
                    <Text
                      style={[
                        style.muliBoldDB,
                        {
                          // flex: 0.5,
                          flexDirection: "column",
                          color: "#10B363",
                          fontSize: 11,
                        },
                      ]}
                    >
                      Upload Done
                    </Text>

                    <Text
                      style={[
                        style.muliboldBB,
                        {
                          flexDirection: "column",
                          color: "#999999",
                          fontSize: 10,
                        },
                      ]}
                    >
                      {item.date}
                    </Text>
                  </View>
                </View>
              </View>
            );
          }
        }}
        contentContainerStyle={{ flex: 0.9, flexDirection: "column" }}
      />
    </View>
  );

  const renderScene = SceneMap({
    first: FirstRoute,
    second: SecondRoute,
  });

  useEffect(async () => {
    if (!interval) {
      interval = setInterval(async () => {
        setTime();
        try {
          MediaStatusSchema = await Database.get("PIMediaStatus");
        } catch (error) {
          console.log("Error in ActivityScreen.js useEffect : ", error);
        }
      }, 1000);
    }
    try {
      MediaStatusSchema = await Database.get("PIMediaStatus");
      if (MediaStatusSchema) {
        MediaStatusSchema = await Database.get("PIMediaStatus");
        console.log(
          "MediaStatusSchema ActivityScreen.js ",
          JSON.stringify(MediaStatusSchema, null, " ")
        );
        var uploadedActivity = MediaStatusSchema.filter((activity) => {
          return activity.activityStatus == "COMPLETED";
        });
        console.log(
          "ActivityScreen.js : COMPLETED ACTIVITY :",
          JSON.stringify(uploadedActivity, null, "  ")
        );
        var allActivity = [];
        var updatedActivityWithProgress = MediaStatusSchema.filter(
          (activity) => {
            var uploadedImage = [];
            var uploadingImages = [];
            var failedImages = [];
            if (activity.activityStatus == "IN-PROGRESS") {
              // not using map,filter becouse require serial exicutions
              for (var uploadingDoc of activity.mediaUpload) {
                if (uploadingDoc.status == "uploading") {
                  uploadingImages.push(uploadingDoc);
                } else if (uploadingDoc.status === "uploaded") {
                  uploadedImage.push(uploadingDoc);
                } else {
                  failedImages.push(uploadingDoc);
                }
              }
              console.log(
                "ActivityScreen.js uploadedImage : ",
                uploadedImage.length,
                " uploadingImages : ",
                uploadingImages.length,
                " failedImages: ",
                failedImages.length
              );
              var progress =
                uploadedImage.length /
                (uploadedImage.length +
                  uploadingImages.length +
                  failedImages.length);
              docProgress = progress;
              console.log("ActivityScreen.js progress : ", progress);
              allActivity.push({
                ...activity,
                imageProgress: isNaN(progress) ? 0 : progress,
              });
              console.log(
                "******* EEEE EEE EE EEEE",
                JSON.stringify(allActivity, null, "  ")
              );
            } else {
              return allActivity.push({
                ...activity,
                imageProgress: isNaN(progress) ? 0 : progress,
              });
            }
          }
        );
        // setting all data in DB
        await Database.set("PIMediaStatus", allActivity);
        const alog = await Database.get("PIMediaStatus");
        console.log(
          "ActivityScreen AfterProgress update PIMediaStatus",
          JSON.stringify(alog, null, "  ")
        );
      }
      unsubscribeNetInfo = NetInfo.addEventListener(async (state) => {
        if (isInternetConnected != null) {
          if (isInternetConnected && (await checkInternetSpeed())) {
            console.log("isInternetConnected is true  in Activityscreen");
            setIsStatus("Uploading");
            SetIsInternetConnected(true);
          } else {
            console.log("isInternetConnected is false in Activityscreen");
            setIsStatus("Upload Paused");
            SetIsInternetConnected(false);
          }
        }
      });
      // const getEstimatedTime = (loaded, total) => {
      //   var seconds_elapsed =
      //     (new Date().getTime() - started_at.getTime()) / 1000;
      //   var bytes_per_second = seconds_elapsed ? loaded / seconds_elapsed : 0;
      //   var Kbytes_per_second = bytes_per_second / 1024;
      //   var remaining_bytes = total - loaded;
      //   var seconds_remaining = seconds_elapsed
      //     ? remaining_bytes / bytes_per_second
      //     : "calculating";
      //   console.log(
      //     "seconds_remaining:: ",
      //     Kbytes_per_second,
      //     seconds_remaining
      //   );
      //   return {
      //     uploadSpeed: Kbytes_per_second,
      //     secondsRemaining: seconds_remaining,
      //   };
      // };
    } catch (error) {
      console.log("Error in useEffect : ", error);
    }
    return () => {
      unsubscribeNetInfo && unsubscribeNetInfo();
    };
  }, [MediaStatusSchema]);
  const setTime = () => {
    const started_at = new Date();

    var seconds_elapsed = (new Date().getTime() - started_at.getTime()) / 1000;
    var bytes_per_second = seconds_elapsed
      ? uploadedArray.length / seconds_elapsed
      : 0;
    var Kbytes_per_second = bytes_per_second / 1024;
    var remaining_bytes = item.mediaUpload.length - uploadedArray.length;
    var seconds_remaining = seconds_elapsed
      ? remaining_bytes / bytes_per_second
      : "calculating";
    console.log("seconds_remaining:: ", Kbytes_per_second, seconds_remaining);
    var time = {
      uploadSpeed: Kbytes_per_second,
      secondsRemaining: seconds_remaining,
    };
    console.log("time:: ****", time);
    setUploadTimeAndSpeed(time);
  };

  const getTimeInMinAndSec = (sec) => {
    console.log("secsecsec", sec);
    if (sec === undefined || sec === null || sec === "calculating") {
      return `0 sec`;
    }

    const min = sec / 60;
    const remSec = sec % 60;
    return `${Math.floor(min)} min ${Math.floor(remSec)} sec`;
  };

  const getSpeedInMbps = (speed) => {
    if (speed === undefined || speed === null) {
      return `0 Mbps`;
    }
    return `${(speed / 1024).toFixed(1)} Mbps`;
  };

  const _renderTabBar = (props) => {
    return (
      <TabBar
        {...props}
        style={{ backgroundColor: "white" }}
        indicatorStyle={{
          borderBottomWidth: 2,
          borderColor: Colors.digitYellow,
        }}
        renderLabel={({ route, color }) => (
          <Text
            style={[
              style.muliBoldDB,
              { fontSize: 14, textTransform: "uppercase" },
            ]}
          >
            {route.title}
          </Text>
        )}
      />
    );
  };

  return (
    <View style={styles.main}>
      <Header
        customTitle={"Activity Status"}
        hideArrowBackButton={false}
        onlanguagechange={() => {
          forceUpdate();
        }}
        onbackPress={() => {
          if (navigation.canGoBack()) {
            navigation.pop();
          } else {
            if (Platform.OS === "android") {
              Alert.alert("Close", "are you sure ?", [
                {
                  text: "Yes",
                  onPress: () => {
                    BackHandler.exitApp();
                  },
                  style: "cancel",
                },
                {
                  text: "No",
                  onPress: () => {
                    console.log("on No pressed");
                  },
                },
              ]);
            }
          }
        }}
      />

      <View style={{ flex: 1, justifyContent: "space-between" }}>
        {isInternetConnected ? null : (
          <View
            style={{
              flex: 0.3,
              flexDirection: "row",
              backgroundColor: "#FFFAEB",
              padding: 12,
              // height: 64,
            }}
          >
            <View style={{ flex: 0.1, flexDirection: "column" }}>
              <MCIcon
                name="wifi-off"
                size={20}
                color="red"
                style={{ marginRight: 4 }}
              />
            </View>
            <View style={{ flex: 0.9, flexDirection: "column" }}>
              <Text style={{ fontSize: 12, fontFamily: "Muli-Bold" }}>
                Poor Network - Uploads paused
                {"\n"}
              </Text>
              <Text style={{ fontSize: 12 }}>
                We'll resume upload when network connection is back. Meanwhile
                you can continue using your phone.
              </Text>
            </View>
          </View>
        )}
        <TabView
          navigationState={{ index, routes }}
          renderScene={renderScene}
          renderTabBar={_renderTabBar}
          onIndexChange={setIndex}
          initialLayout={{ width: layout.width }}
          style={{ flex: 0.9 }}
        />
        <View
          style={{
            flex: 0.1,
            marginHorizontal: 16,
            justifyContent: "center",
          }}
        >
          <PrimaryButton
            onPress={async () => {
              interval && clearInterval(interval);
              navigation.pop();
            }}
          >
            {"Go to Homepage"}
          </PrimaryButton>
        </View>
      </View>
    </View>
  );
};

export default ActivityScreen;

const styles = StyleSheet.create({
  main: {
    flex: 1,
    backgroundColor: "#FFFFFF",
  },
  subView: {
    margin: 18,
    flex: 1,
  },
  ThankYoumsgView: {
    flex: 0.95,
  },
  needHelpView: {
    flex: 0.15,
    marginTop: 48,
  },
  thankyoumsg: {
    marginLeft: 6,
    marginRight: 6,
    marginTop: 33,
    flex: 1,
  },
  allDoneText: {
    fontFamily: "Muli-Black",
    fontSize: 20,
  },
  PImsg: {
    fontSize: 16,
    fontFamily: "Muli-SemiBold",
    marginTop: 8,
  },
  needHelpText: {
    fontFamily: "Muli-ExtraBold",
    fontSize: 12,
    color: "#999999",
  },
  buttonStyle: {
    height: 40,
    marginLeft: 6,
    marginRight: 6,
    borderColor: "#444444",
    borderWidth: 1,
    borderRadius: 4,
    marginTop: 12,
    flexDirection: "row",
    alignItems: "center",
    marginBottom: 4,
  },
  buttonText: {
    fontFamily: "Muli-Bold",
    fontSize: 16,
    marginLeft: 16,
  },
  tabBar: {
    flexDirection: "row",
  },
  tabItem: {
    flex: 1,
    alignItems: "center",
    padding: 16,
  },
});
