import React, { useEffect, useRef, useState } from "react";
import {
  StyleSheet,
  View,
  Text,
  TouchableOpacity,
  Image,
  Button,
  LogBox,
  StatusBar,
  Linking,
  BackHandler,
  useWindowDimensions,
  Dimensions,
  FlatList,
  Alert,
} from "react-native";
import Header from "react-native-digitcamera/src/Screens/Header";
// import FirstRoute from "./FirstRoute";
// import SecondRoute from "./SecondRoute";
import { TabView, SceneMap } from "react-native-tab-view";
import {
  TextInput,
  PrimaryButton,
  NewRoundButton,
  Spinner,
  NewRadioButton,
} from "../../common/ThemedComponents";
import NetInfo from "@react-native-community/netinfo";
import { checkInternetSpeed } from "react-native-digitcamera/src/Screens/OfflineFlow/CommonFunc";
import { useIsFocused } from "@react-navigation/native";
import MCIcon from "react-native-vector-icons/MaterialCommunityIcons";
import { ProgressBar } from "react-native-paper";
import Toturial from "react-native-digitcamera/src/Screens/Toturial";
import Database from "../../../../../Components/common/DAsyncStorage";
import { ScrollView } from "react-native-gesture-handler";
import { async } from "validate.js";

//Constants
const SCREEN_WIDTH = Math.round(Dimensions.get("window").width);
const SCREEN_HEIGHT = Math.round(Dimensions.get("window").height);

var totalLength = 0;
var inProgressLength = 0;
var inProgressArray = [];
var completedArray = [];
var name = "";
var mediaStatusArray = [];
let interval;
const ActivityStatusScreen = ({ navigation, route }) => {
  var isFocused = useIsFocused();
  console.log("isFocused : ", isFocused);
  const layout = useWindowDimensions();
  const [isInternetConnected, SetIsInternetConnected] = useState(true);
  const [isStatus, setIsStatus] = useState(true);
  const [docProgress, setDocProgress] = useState(0);

  const [index, setIndex] = React.useState(0);
  const [uploadTimeAndSpeed, setUploadTimeAndSpeed] = useState({
    uploadSpeed: 0,
    secondsRemaining: 0,
  });

  useEffect(async () => {
    var CustomerDetails = await Database.get("CustomerDetails");
    name = CustomerDetails?.name;
    if (interval) {
      console.log("*** ActivityStatusScreen interval already available");
      clearInterval(interval);
    }
    interval = setInterval(() => {
      const check = async () => {
        console.log("ActivityStatusScreen.js in check interval");
        updateData();
      };
      check();
    }, 1000);

    return () => {
      unsubscribeNetInfo();
    };
  }, [isFocused]);

  const updateData = async () => {
    try {
      mediaStatusArray = await Database.get("PIMediaStatus");
      inProgressArray = [];
      const unsubscribeNetInfo = NetInfo.addEventListener(async (state) => {
        if (isInternetConnected != null) {
          if (isInternetConnected && (await checkInternetSpeed())) {
            console.log("isInternetConnected is true in Activitystatusscreen");
            setIsStatus("Uploading");
            SetIsInternetConnected(true);
          } else {
            console.log("isInternetConnected is false in Activitystatusscreen");
            setIsStatus("Upload Paused");
            SetIsInternetConnected(false);
          }
        }
      });
      console.log(
        "mediaStatusArray******* ",
        JSON.stringify(mediaStatusArray, null, " ")
      );
      totalLength = mediaStatusArray.length;
      completedArray = [];
      inProgressArray = [];
      for (var item of mediaStatusArray) {
        console.log("mediaStatusArray item &&&&&& ssss", JSON.stringify(item));
        if (item.activityStatus == "IN-PROGRESS") {
          inProgressArray.push(item);
        }
        if (item.activityStatus == "COMPLETED") {
          completedArray.push(item);
        }
      }
      console.log(
        "ActivityStatusScreen.js inProgressArray : ",
        inProgressArray.length
      );
      console.log(
        "ActivityStatusScreen.js completedArray : ",
        completedArray.length
      );

      inProgressLength = inProgressArray.length;
      var tempDocProgress =
        completedArray.length /
        (completedArray.length + inProgressArray.length);
      setDocProgress(tempDocProgress);
      console.log("docProgress activitystatusscreen:::: ", docProgress);

      const started_at = new Date();

      var seconds_elapsed =
        (new Date().getTime() - started_at.getTime()) / 1000;
      var bytes_per_second = seconds_elapsed
        ? totalLength / seconds_elapsed
        : 0;
      var Kbytes_per_second = bytes_per_second / 1024;
      var remaining_bytes = inProgressLength - totalLength;
      var seconds_remaining = seconds_elapsed
        ? remaining_bytes / bytes_per_second
        : "calculating";
      console.log("seconds_remaining:: ", Kbytes_per_second, seconds_remaining);
      var time = {
        uploadSpeed: Kbytes_per_second,
        secondsRemaining: seconds_remaining,
      };
      console.log("time:: ****", time);
      setUploadTimeAndSpeed(time);
    } catch (error) {
      console.log("error in activitystatuscreen.js updateData() ", error);
    }
  };
  const getTimeInMinAndSec = (sec) => {
    console.log("secsecsec", sec);
    if (sec === undefined || sec === null || sec === "calculating") {
      return `0 sec`;
    }

    const min = sec / 60;
    const remSec = sec % 60;
    return `${Math.floor(min)} min ${Math.floor(remSec)} sec`;
  };

  const getSpeedInMbps = (speed) => {
    if (speed === undefined || speed === null) {
      return `0 Mbps`;
    }
    return `${(speed / 1024).toFixed(3)} Mbps`;
  };

  const onHandleBtnPress = async () => {
    console.log("*** ActivityStatusScreen interval onHandleBtnPress");
    clearInterval(interval);
    console.log("on handle btn press");
    // navigate to WithQuote
    await Database.update("PICanShowActivity", {
      isShowActivityScreen: false,
    });
    navigation.navigate("WithQuote");
  };

  return (
    <View style={[styles.main, { backgroundColor: "#EAEAEA" }]}>
      <Header
        customTitle={"Activity Status"}
        hideArrowBackButton={false}
        onlanguagechange={() => {
          forceUpdate();
        }}
        onbackPress={() => {
          if (navigation.canGoBack()) {
            navigation.pop();
          } else {
            if (Platform.OS === "android") {
              Alert.alert("Close", "are you sure ?", [
                {
                  text: "Yes",
                  onPress: () => {
                    BackHandler.exitApp();
                  },
                  style: "cancel",
                },
                {
                  text: "No",
                  onPress: () => {
                    console.log("on No pressed");
                  },
                },
              ]);
            }
          }
        }}
      />
      <ScrollView>
        <View
          style={{
            flex: 0.2,
            flexDirection: "row",
            padding: 16,
            backgroundColor: "white",
          }}
        >
          <View
            style={{
              flex: 1,
              flexDirection: "column",
            }}
          >
            <Text style={styles.heading}>
              Welcome {name},{"\n"}
            </Text>
            <Text style={styles.paragraph}>You can shoot video of car</Text>
          </View>
        </View>
        <View
          style={{
            flex: 0.8,
            flexDirection: "row",
          }}
        >
          <View style={{ flex: 1, flexDirection: "column" }}>
            <View style={[{ flex: 0.6, flexDirection: "row" }, styles.box]}>
              <View style={{ flex: 1, flexDirection: "column" }}>
                <View
                  style={{
                    flex: 0.6,
                    flexDirection: "row",
                    width: "100%",
                    height: SCREEN_HEIGHT / 4,
                    // backgroundColor: "red",
                  }}
                >
                  <Image
                    source={{
                      uri: "https://d2h44aw7l5xdvz.cloudfront.net/CRM/PRM/selfie_car.png",
                    }}
                    resizeMode={"contain"}
                    style={{ width: "100%", height: "100%" }}
                  />
                </View>
                <View style={{ flex: 0.4, flexDirection: "row", padding: 5 }}>
                  <View style={{ flex: 1, flexDirection: "column" }}>
                    <View style={{ flex: 0.33, flexDirection: "row" }}>
                      <Text style={styles.heading}>Shoot Video</Text>
                    </View>
                    <View
                      style={{ flex: 0.33, flexDirection: "row", padding: 5 }}
                    >
                      {/* <Text>
                      Lorem ipsum scing elit. Suspend isse justo, facilisis eget
                      porta.
                    </Text> */}
                    </View>
                    <View
                      style={{
                        flex: 0.33,
                        flexDirection: "row",
                        marginBottom: 12,
                      }}
                    >
                      <PrimaryButton onPress={onHandleBtnPress}>
                        {"Start"}
                      </PrimaryButton>
                    </View>
                  </View>
                </View>
              </View>
            </View>

            <View
              style={[
                {
                  flex: 0.2,
                  flexDirection: "row",
                },
                styles.box,
              ]}
            >
              <TouchableOpacity
                style={{ flex: 1, flexDirection: "column" }}
                onPress={() => {
                  console.log(
                    "*** ActivityStatusScreen interval touchable opacity before activityScreen"
                  );
                  clearInterval(interval);
                  /* Don't know what is this need more clearance */
                  navigation.navigate("ActivityScreen");
                }}
              >
                <View style={{ flex: 0.5, flexDirection: "row" }}>
                  <Text style={{ fontFamily: "Muli-Bold", fontSize: 16 }}>
                    Activity Status
                  </Text>
                </View>
                {inProgressLength > 0 ? (
                  <View style={{ flex: 0.5, flexDirection: "row" }}>
                    <View style={{ flex: 1, flexDirection: "column" }}>
                      <ProgressBar
                        progress={
                          isNaN(docProgress) ||
                          docProgress == undefined ||
                          docProgress == null
                            ? 0
                            : docProgress
                        }
                        color={"#FFBB00"}
                        style={{
                          height: 4,
                          borderRadius: 12,
                          backgroundColor: "#F5F5F5",
                        }}
                      />

                      <View
                        style={{
                          flex: 0.5,
                          flexDirection: "row",
                          marginTop: 2,
                        }}
                      >
                        <View style={{ flex: 1, flexDirection: "column" }}>
                          <View style={{ flex: 1, flexDirection: "row" }}>
                            <Text
                              style={{
                                flex: 0.33,
                                flexDirection: "column",
                              }}
                            >
                              {inProgressLength} Uploading
                            </Text>
                            <MCIcon
                              name="wifi"
                              size={16}
                              color="#EE0000"
                              style={{ marginRight: 4 }}
                            />
                            <Text
                              style={{
                                flex: 0.33,
                                flexDirection: "column",
                              }}
                            >
                              {getSpeedInMbps(uploadTimeAndSpeed?.uploadSpeed)}
                            </Text>
                            <Text
                              style={{
                                flex: 0.33,
                                flexDirection: "column",
                                fontStyle: "italic",
                              }}
                            >
                              {getTimeInMinAndSec(
                                uploadTimeAndSpeed?.secondsRemaining
                              )}{" "}
                              left
                            </Text>
                          </View>
                        </View>
                      </View>
                    </View>
                  </View>
                ) : (
                  <View style={{ flex: 0.5, flexDirection: "row" }}>
                    <View
                      style={{
                        flex: 1,
                        flexDirection: "column",
                        paddingTop: 8,
                      }}
                    >
                      <Text
                        style={{
                          color: "#444444",
                          fontSize: 13,
                          fontFamily: "Muli-Regular",
                        }}
                      >
                        You can view video upload status here
                      </Text>

                      <View
                        style={{
                          flex: 0.5,
                          flexDirection: "row",
                          marginTop: 2,
                        }}
                      >
                        <View
                          style={{
                            flex: 1,
                            flexDirection: "column",
                            paddingTop: 8,
                          }}
                        >
                          <View style={{ flex: 1, flexDirection: "row" }}>
                            <Text
                              style={{
                                flex: 0.33,
                                flexDirection: "column",
                                fontFamily: "Muli-Bold",
                                fontSize: 11,
                              }}
                            >
                              No Uploads
                            </Text>
                          </View>
                        </View>
                      </View>
                    </View>
                  </View>
                )}

                <View style={{ flex: 0.33, flexDirection: "row" }} />
              </TouchableOpacity>
            </View>

            <View
              style={{
                flex: 0.2,
                flexDirection: "row",
              }}
            />
          </View>
        </View>
      </ScrollView>
    </View>
  );
};
export default ActivityStatusScreen;

const styles = StyleSheet.create({
  main: {
    flex: 1,
    backgroundColor: "#FFFFFF",
  },
  box: {
    borderWidth: 1,
    borderColor: "#DDDDDD",
    margin: 12,
    padding: 12,
    shadowColor: "#DDDDDD",
    shadowOpacity: 0.16,
    borderRadius: 8,
    backgroundColor: "#ffffff",
  },
  paragraph: {
    fontSize: 18,

    // textAlign: "center",
  },
  heading: {
    fontSize: 22,
    alignSelf: "flex-start",
    fontFamily: "Muli-Bold",
  },
  subView: {
    margin: 18,
    flex: 1,
  },
  ThankYoumsgView: {
    flex: 0.95,
  },
  needHelpView: {
    flex: 0.15,
    marginTop: 48,
  },
  thankyoumsg: {
    marginLeft: 6,
    marginRight: 6,
    marginTop: 33,
    flex: 1,
  },
  allDoneText: {
    fontFamily: "Muli-Black",
    fontSize: 20,
  },
  PImsg: {
    fontSize: 16,
    fontFamily: "Muli-SemiBold",
    marginTop: 8,
  },
  needHelpText: {
    fontFamily: "Muli-ExtraBold",
    fontSize: 12,
    color: "#999999",
  },
  buttonStyle: {
    height: 40,
    marginLeft: 6,
    marginRight: 6,
    borderColor: "#444444",
    borderWidth: 1,
    borderRadius: 4,
    marginTop: 12,
    flexDirection: "row",
    alignItems: "center",
    marginBottom: 4,
  },
  buttonText: {
    fontFamily: "Muli-Bold",
    fontSize: 16,
    marginLeft: 16,
  },
});
